"""Checker for naming conventions"""
import discord
from .basechecker import DiscordBaseChecker
from pylint.interfaces import INFERENCE

class NamingConventionChecker(DiscordBaseChecker):
    """Checker for naming conventions"""

    name = "naming_convention"
    msgs = {
        "C9006": (
            "A bot instance should not be named client",
            "misleading-client",
            "You should not name your bot instance, client since it's misleading",
        ),
    }

    def visit_assign(self, node):
        if node.targets and node.targets[0] and node.targets[0].name == "client":
            if node.value and node.value.func:
                if hasattr(node.value.func, 'name') and "bot" in node.value.func.name.lower():
                    self.add_message(
                        "C9006",
                        node=node,
                        args=(),
                        confidence=INFERENCE
                    )
                if hasattr(node.value.func, 'expr') and node.value.func.expr.name == "commands":
                    self.add_message(
                        "C9006",
                        node=node,
                        args=(),
                    )






"""Discord.py Linter"""
from .checkers import HasPermissionsChecker, EventsChecker, NamingConventionChecker


def register(linter):
    """Registers the linters to pylint"""
    linter.register_checker(HasPermissionsChecker(linter))
    linter.register_checker(EventsChecker(linter))
    linter.register_checker(NamingConventionChecker(linter))

__name__    = "dpylint"
__title__   =  __name__
__license__ = "MIT"
__version__ = "0.2.2"
__author__  = "Arian Mollik Wasi"
__github__  = "https://github.com/wasi-master/dpylint"

__all__ = ("register",)



from .has_permissions_checker import HasPermissionsChecker
from .event_checker import EventsChecker
from .naming_convention_checker import NamingConventionChecker

__all__ = ("HasPermissionsChecker", "EventsChecker","NamingConventionChecker")



import dpylint
import pylint.testutils
import pylint.interfaces
import astroid


class TestNamingConventionChecker(pylint.testutils.CheckerTestCase):
    CHECKER_CLASS = dpylint.NamingConventionChecker

    def test_misleading_client_name(self):
        assign_node= astroid.extract_node("""
        client = commands.Bot("!")
        """)

        self.checker.visit_assign(assign_node)
        with self.assertAddsMessages(
            pylint.testutils.MessageTest(
                msg_id='C9006',
                node=assign_node,
                args=()
            )
        ):
            pass

    def test_misleading_client_name_with_custom_class(self):
        assign_node= astroid.extract_node("""
        class MyBot(commands.Bot): pass
        client = MyBot("!")
        """)

        self.checker.visit_assign(assign_node)
        with self.assertAddsMessages(
            pylint.testutils.MessageTest(
                msg_id='C9006',
                node=assign_node,
                confidence=pylint.interfaces.INFERENCE,
                args=()
            )
        ):
            pass

